use sha1_smol::Sha1;

const BLOCK_SIZE: usize = 64;

pub fn hmac_sha1(key: &[u8], message: &[u8], output: &mut [u8]) {
    // Preprocess the key.
    let mut key_padded = [0u8; BLOCK_SIZE];
    if key.len() > BLOCK_SIZE {
        let mut hash = Sha1::new();
        hash.update(key);
        let key_digest = hash.digest().bytes();
        key_padded[..20].copy_from_slice(&key_digest);
    } else {
        key_padded[..key.len()].copy_from_slice(key);
    }
    for byte in &mut key_padded {
        *byte ^= 0x36;
    }

    // Hash the inner padded key and the message.
    let mut hash = Sha1::new();
    hash.update(&key_padded[..]);
    hash.update(message);
    let mut inner_hash = [0u8; 20];
    inner_hash.copy_from_slice(&hash.digest().bytes());

    // Preprocess the key again.
    for byte in &mut key_padded {
        *byte ^= 0x36 ^ 0x5c;
    }

    // Hash the outer padded key and the inner hash.
    let mut hash = Sha1::new();
    hash.update(&key_padded[..]);
    hash.update(&inner_hash[..]);
    let hmac_digest = hash.digest().bytes();

    // Copy the result into the output buffer.
    output.copy_from_slice(&hmac_digest);
}
